/* eslint-disable import/extensions */
require('dotenv').config()

const pkg = require('./package.json')

const productName = pkg.productName

console.log(`\n Building ${productName} for ${pkg.version} Version`)

const publishers = [
  {
    name: '@electron-forge/publisher-github',
    config: {
      repository: {
        owner: 'Iconscout',
        name: 'iconscout-app',
      },
      draft: true,
      prerelease: true,
      authToken: process.env.GITHUB_TOKEN,
    },
  },
]

module.exports = {
  packagerConfig: {
    icon: './icons/Iconscout',
    osxSign: {
      identity: 'Developer ID Application: Design Barn Inc. (VT33U5MQ8Q)',
      'hardened-runtime': true,
      'gatekeeper-assess': false,
      entitlements: './build/default.entitlements.darwin.plist',
      'entitlements-inherit':
        './build/default.entitlements.darwin.inherit.plist',
      timestamp: 'http://timestamp.apple.com/ts01',
    },
  },
  hooks: {
    postPackage: require('./scripts/notarize.js'),
  },
  makers: [
    {
      name: '@electron-forge/maker-dmg',
      config: {
        format: 'ULFO',
      },
    },
    {
      name: '@electron-forge/maker-zip',
    },
    {
      name: '@electron-forge/maker-squirrel',
      config: {
        setupIcon: './icons/Iconscout.ico',
        loadingGif: './build/installer.gif',
        certificateFile: process.env.CODE_SIGN_PATH || './certs/windows.pfx',
        certificatePassword: process.env.CODE_SIGN_PASS,
      },
    },
    {
      name: '@electron-forge/maker-deb',
      config: {
        options: {
          bin: productName,
        },
      },
    },
  ],
  publishers,
}
