Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@sentry/core");
exports.CommonBackend = core_1.BaseBackend;
const utils_1 = require("@sentry/utils");
/** IPC to ping the main process when initializing in the renderer. */
exports.IPC_PING = 'sentry-electron.ping';
/** IPC to send a captured event (exception or message) to Sentry. */
exports.IPC_EVENT = 'sentry-electron.event';
/** IPC to capture a scope globally. */
exports.IPC_SCOPE = 'sentry-electron.scope';
/** Get the name of an electron app for <v5 and v7< */
function getNameFallback() {
    // tslint:disable-next-line: strict-type-predicates
    if (!require) {
        throw new utils_1.SentryError('Could not require("electron") to get appName. Please ensure you pass `appName` to Sentry options');
    }
    const electron = require('electron');
    // if we're in the main process
    if (electron && electron.app) {
        const appMain = electron.app;
        return appMain.name || appMain.getName();
    }
    // We're in the renderer process but the remote module is not available
    if (!electron || !electron.remote) {
        throw new utils_1.SentryError('The Electron `remote` module was not available to get appName. Please ensure you pass `appName` to Sentry options');
    }
    // Remote is available so get the app name
    const a = electron.remote.app;
    return a.name || a.getName();
}
exports.getNameFallback = getNameFallback;
//# sourceMappingURL=common.js.map