import { BaseBackend } from '@sentry/core';
import { Event, EventHint, Severity, Transport } from '@sentry/types';
import { CommonBackend, ElectronOptions } from '../common';
/**
 * Retruns a promise that resolves when app is ready.
 */
export declare function isAppReady(): Promise<boolean>;
/** Backend implementation for Electron renderer backends. */
export declare class MainBackend extends BaseBackend<ElectronOptions> implements CommonBackend<ElectronOptions> {
    /** The inner SDK used to record Node events. */
    private readonly _inner;
    /** Store to persist context information beyond application crashes. */
    private readonly _scopeStore;
    /** Uploader for minidump files. */
    private _uploader?;
    /** Creates a new Electron backend instance. */
    constructor(options: ElectronOptions);
    /**
     * Setup Transport
     */
    protected _setupTransport(): Transport;
    /**
     * @inheritDoc
     */
    eventFromException(exception: any, hint?: EventHint): PromiseLike<Event>;
    /**
     * @inheritDoc
     */
    eventFromMessage(message: string, level?: Severity, hint?: EventHint): PromiseLike<Event>;
    /**
     * Uploads the given minidump and attaches event information.
     *
     * @param path A relative or absolute path to the minidump file.
     * @param event Optional event information to add to the minidump request.
     */
    uploadMinidump(path: string, event?: Event): void;
    /**
     * Loads the stored scope from disk ands sets it int the current scope
     */
    private _rehydrateScope;
    /**
     * Adds a scope listener to persist changes to disk.
     */
    private _setupScopeListener;
    /** Returns whether native reports are enabled. */
    private _isNativeEnabled;
    /** Activates the Electron CrashReporter. */
    private _installNativeHandler;
    /** Installs IPC handlers to receive events and metadata from renderers. */
    private _installIPC;
    /** Loads new native crashes from disk and sends them to Sentry. */
    private _sendNativeCrashes;
    /** Returns extra information from a renderer's web contents. */
    private _getRendererExtra;
}
//# sourceMappingURL=backend.d.ts.map