require('dotenv').config()

const { app, BrowserWindow } = require('electron')

const path = require('path')

const { channel } = require('./package')

// Check for updates if App is in Beta or Production Mode
if (channel !== 'alpha') {
  require('update-electron-app')({
    repo: 'Iconscout/iconscout-app',
  })
} else {
  console.info(`aborting updates since app is in ${channel} channel`)
}

require('./listeners/item')

// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (require('electron-squirrel-startup')) {
  // eslint-disable-line global-require
  app.quit()
}

require('./sentry')
require('./auto-launch')

const indexFile =
  process.env.NODE_ENV === 'development'
    ? 'http://localhost:8080/'
    : `file://${__dirname}/html/index.html`
const browserWindow = {
  width: 380,
  height: 500,
  maximizable: false,
  autoHideMenuBar: true,
  webPreferences: {
    nodeIntegration: true,
    preload: path.join(__dirname, 'app.js'),
  },
}

const isWin = process.platform === 'win32'

if (isWin) {
  // If Windows then show it as a desktop app
  const createWindow = () => {
    const win = new BrowserWindow({
      ...browserWindow,
      alwaysOnTop: true,
    })

    // and load the index.html of the app.
    win.loadURL(indexFile)

    // Open the DevTools.
    if (process.env.NODE_ENV === 'development') {
      win.webContents.openDevTools()
    }
  }

  app.whenReady().then(createWindow)
} else {
  // For Mac and Linux show it in the top menubar
  const { menubar } = require('menubar')
  const mb = menubar({
    index: indexFile,
    browserWindow,
    // Disabling because it was causing issue in Login with Web feature.
    preloadWindow: process.env.NODE_ENV === 'development',
  })

  mb.on('after-create-window', () => {
    if (process.env.NODE_ENV === 'development') {
      mb.window.openDevTools()
    }
  })
}
