const fs = require('fs')
const path = require('path')

const { app, ipcMain, shell } = require('electron')
const rimraf = require('rimraf')
const { v5: uuidv5 } = require('uuid')

const download = require('../helpers/download')
const getFileName = require('../helpers/filename')
const rename = require('../helpers/rename')

const tempFolder = path.join(
  app.getPath('temp'),
  uuidv5('IconScout', uuidv5.URL),
)

if (!fs.existsSync(tempFolder)) {
  fs.mkdirSync(tempFolder)
}

console.log('temp', tempFolder)

ipcMain.handle('download', async (event, url) => {
  try {
    const filename = getFileName(url)
    await download(url, path.join(tempFolder, filename))
    return true
  } catch (error) {
    console.error(error)
    return false
  }
})

ipcMain.on('dragstart', async (event, { name, url }) => {
  const filename = getFileName(url)
  const source = path.join(tempFolder, filename)
  const ext = path.extname(source).replace('.', '')

  // Confirm file is downloaded
  await download(url, source)
  // Rename file with Name so the item gets inserted properly
  const file = await rename(tempFolder, filename, name + '.' + ext)

  event.sender.startDrag({
    file,
    icon: path.join(__dirname, `../images/${ext}.png`),
  })
})

ipcMain.on('openURL', async (event, url) => {
  shell.openExternal(url)
})

ipcMain.on('quit', async () => {
  rimraf.sync(tempFolder)
  app.quit()
})
